AuthenticatorError

Enum AuthenticatorError 

pub enum AuthenticatorError {
Show 23 variants PrimitiveError(PrimitiveError), AccountDoesNotExist, ContractError(Error), NetworkError(Error), PublicKeyNotFound, GatewayError { status: StatusCode, body: String, }, IndexerError { status: StatusCode, body: String, }, Timeout, InvalidConfig { attribute: String, reason: String, }, InvalidCredentialForProofRequest, UnfullfilableRequest, InvalidSubOrBlindingFactor, RegistrationError { error_code: String, error_message: String, }, ProofError(ProofError), InvalidIndexerPubkeySlot { slot_index: usize, max_supported_slot: usize, }, OhttpEncapsulationError(Error), BhttpError(Error), OhttpRelayError { status: StatusCode, body: String, }, InvalidServiceResponse(String), ResponseValidationError(ValidationError), ProofMaterialsNotLoaded, SessionIdMismatch, Generic(String),
}
Errors that can occur when interacting with the Authenticator.

Variants



PrimitiveError(PrimitiveError)

Primitive error



AccountDoesNotExist

This operation requires a registered account and an account is not registered for this authenticator. Call create_account first to register it.



ContractError(Error)

An error occurred while interacting with the EVM contract.



NetworkError(Error)

Network/HTTP request error.



PublicKeyNotFound

Public key not found in the Authenticator public key set. Usually indicates the local state is out of sync with the registry.



GatewayError

Gateway returned an error response.

Fields

status: StatusCode

HTTP status code

body: String

Response body



IndexerError

Indexer returned an error response.

Fields

status: StatusCode

HTTP status code

body: String

Response body



Timeout

Account creation timed out while polling for confirmation.



InvalidConfig

Configuration is invalid or missing required values.

Fields

attribute: String

The config attribute that is invalid.

reason: String

Description of why it is invalid.



InvalidCredentialForProofRequest

The provided credential is not valid for the provided proof request.



UnfullfilableRequest

The provided credentials do not satisfy the proof request.

This usually means the authenticator made an incorrect selection of credentials.



InvalidSubOrBlindingFactor

Used in Ownership Proofs. The provided blinding_factor does not match for the provided sub, ownership cannot be proven.



RegistrationError

Error during the World ID registration process.

This usually occurs from an on-chain revert.

Fields

error_code: String

Error code from the registration process.

error_message: String

Detailed error message.



ProofError(ProofError)

Error on proof generation



InvalidIndexerPubkeySlot

Indexer returned an authenticator key slot that exceeds supported key capacity.

Fields

slot_index: usize

Slot index returned by the indexer.

max_supported_slot: usize

Highest supported slot index.



OhttpEncapsulationError(Error)

OHTTP encapsulation or decapsulation error.



BhttpError(Error)

Binary HTTP framing error.



OhttpRelayError

The OHTTP relay itself returned a non-success status.

Fields

status: StatusCode

HTTP status code from the relay.

body: String

Response body from the relay.



InvalidServiceResponse(String)

A service returned a success status but the response body could not be deserialized into the expected type.



ResponseValidationError(ValidationError)

The assembled proof response failed self-validation against the request.



ProofMaterialsNotLoaded

Proof materials not loaded. Call with_proof_materials before generating proofs.



SessionIdMismatch

The session ID computed for this proof does not match the expected session ID from the proof request.

This indicates the session_id provided by the RP is invalid or compromised, or the authenticator cached the wrong session_id_r_seed for the oprf_seed.



Generic(String)

Generic error for other unexpected issues.