pub struct Config { /* private fields */ }Expand description
Global configuration to interact with the different components of the Protocol.
Used by Authenticators and RPs.
Implementations
Sourceimpl Config
impl Config
Sourcepub fn new(
rpc_url: Option<String>,
chain_id: u64,
registry_address: Address,
indexer_url: String,
gateway_url: String,
nullifier_oracle_urls: Vec<String>,
nullifier_oracle_threshold: usize,
) -> Result<Config, PrimitiveError>
pub fn new( rpc_url: Option<String>, chain_id: u64, registry_address: Address, indexer_url: String, gateway_url: String, nullifier_oracle_urls: Vec<String>, nullifier_oracle_threshold: usize, ) -> Result<Config, PrimitiveError>
Sourcepub const fn chain_id(&self) -> u64
pub const fn chain_id(&self) -> u64
The chain ID of the network where the WorldIDRegistry contract is deployed.
Sourcepub const fn registry_address(&self) -> &Address
pub const fn registry_address(&self) -> &Address
The address of the WorldIDRegistry contract.
Sourcepub const fn indexer_url(&self) -> &String
pub const fn indexer_url(&self) -> &String
The URL of the world-id-indexer service to use. The indexer is used to fetch inclusion proofs from the WorldIDRegistry contract.
Sourcepub const fn gateway_url(&self) -> &String
pub const fn gateway_url(&self) -> &String
The URL of the world-id-gateway service to use. The gateway is used to perform operations on the WorldIDRegistry contract
without leaking a wallet address.
Sourcepub const fn nullifier_oracle_urls(&self) -> &Vec<String>
pub const fn nullifier_oracle_urls(&self) -> &Vec<String>
The list of URLs of all and each node of the Nullifier Oracle.
Sourcepub const fn nullifier_oracle_threshold(&self) -> usize
pub const fn nullifier_oracle_threshold(&self) -> usize
The minimum number of Nullifier Oracle responses required to build a nullifier.