pub struct ProofResponse {
pub id: String,
pub version: RequestVersion,
pub session_id: Option<SessionId>,
pub error: Option<String>,
pub responses: Vec<ResponseItem>,
}Expand description
Overall response from the Authenticator to the RP
Fields
id: StringThe response id references request id
version: RequestVersionVersion corresponding to request version
session_id: Option<SessionId>RP session identifier that links multiple proofs for the same user/RP pair across requests.
For an initial request which creates a session, this contains
the newly generated SessionId. For subsequent Session Proofs, this
echoes back the SessionId from the request for convenience.
This is optional as it’s not provided in Uniqueness Proofs.
error: Option<String>Error message if the entire proof request failed. When present, the responses array will be empty.
responses: Vec<ResponseItem>Per-credential results (empty if error is present)
Implementations
Sourceimpl ProofResponse
impl ProofResponse
Sourcepub fn constraints_satisfied(&self, constraints: &ConstraintExpr<'_>) -> bool
pub fn constraints_satisfied(&self, constraints: &ConstraintExpr<'_>) -> bool
Determine if constraints are satisfied given a constraint expression. Returns false if the response has an error.
Sourceimpl ProofResponse
impl ProofResponse
Sourcepub fn to_json_pretty(&self) -> Result<String, Error>
pub fn to_json_pretty(&self) -> Result<String, Error>
Sourcepub fn successful_credentials(&self) -> Vec<u64>
pub fn successful_credentials(&self) -> Vec<u64>
Return the list of successful issuer_schema_ids in the response.
Returns an empty vec if the response has an error.