pub enum ValidationError {
RequestIdMismatch,
VersionMismatch,
MissingCredential(String),
ConstraintNotSatisfied,
ConstraintTooDeep,
ConstraintTooLarge,
}Expand description
Validation errors when checking a response against a request
Variants
RequestIdMismatch
The response id does not match the request id
VersionMismatch
The response version does not match the request version
MissingCredential(String)
A required credential was not provided
ConstraintNotSatisfied
The provided credentials do not satisfy the request constraints
ConstraintTooDeep
The constraints expression exceeds the supported nesting depth
ConstraintTooLarge
The constraints expression exceeds the maximum allowed size/complexity