RequestItem

Struct RequestItem 

pub struct RequestItem {
    pub identifier: String,
    pub issuer_schema_id: FieldElement,
    pub signal: Option<String>,
    pub genesis_issued_at_min: Option<u64>,
    pub session_id: Option<FieldElement>,
}
Per-credential request payload

Fields

identifier: String

An RP-defined identifier for this request item which can be used to match against constraints and responses.

Example: orb, document.

issuer_schema_id: FieldElement

The specific credential being requested as registered in the CredentialIssuerSchemaRegistry. Serialized as hex string in JSON.

signal: Option<String>

Optional RP-defined signal that will be bound into the proof.

When present, the authenticator hashes this via signal_hash and commits it into the proof circuit so the RP can tie the proof to a particular action.

genesis_issued_at_min: Option<u64>

An optional constraint on the minimum genesis issued at timestamp on the used credential.

If present, the proof will include a constraint that the credential’s genesis issued at timestamp is greater than or equal to this value. This is useful for migration from previous protocol versions.

session_id: Option<FieldElement>

If provided, a Session Proof will be generated instead of a Uniqueness Proof.

The proof will only be valid if the session ID is meant for this context and this particular World ID holder.

Implementations

Source

impl RequestItem

pub const fn new( identifier: String, issuer_schema_id: FieldElement, signal: Option<String>, genesis_issued_at_min: Option<u64>, session_id: Option<FieldElement>, ) -> Self

Create a new request item with the given identifier, issuer schema ID and optional signal.

pub fn signal_hash(&self) -> FieldElement

Get the signal hash for the request item.