pub struct EdDSAPrivateKey(/* private fields */);Expand description
A private key for the EdDSA signature scheme.
Implementations
Sourceimpl EdDSAPrivateKey
impl EdDSAPrivateKey
Sourcepub fn from_bytes(bytes: [u8; 32]) -> EdDSAPrivateKey
pub fn from_bytes(bytes: [u8; 32]) -> EdDSAPrivateKey
Create a private key from a 32-byte array.
Sourcepub fn random<R>(rng: &mut R) -> EdDSAPrivateKey
pub fn random<R>(rng: &mut R) -> EdDSAPrivateKey
Generate a random private key using the given RNG.
Sourcepub fn public(&self) -> EdDSAPublicKey
pub fn public(&self) -> EdDSAPublicKey
Derive the public key corresponding to this private key.
Sourcepub fn sign(&self, message: Fp<MontBackend<FrConfig, 4>, 4>) -> EdDSASignature
pub fn sign(&self, message: Fp<MontBackend<FrConfig, 4>, 4>) -> EdDSASignature
Sign a message (a BaseField element) with the given secret key (a ScalarField element).
The message should be hashed to a BaseField element if it is not encodable as one before signing.