pub struct InitializingAuthenticator { /* private fields */ }Expand description
Represents an account in the process of being initialized,
i.e. it is not yet registered in the WorldIDRegistry contract.
Implementations
Sourceimpl InitializingAuthenticator
impl InitializingAuthenticator
Sourcepub fn request_id(&self) -> &str
pub fn request_id(&self) -> &str
Returns the gateway request ID for this pending account creation.
Sourcepub async fn poll_status(
&self,
) -> Result<GatewayRequestState, AuthenticatorError>
pub async fn poll_status( &self, ) -> Result<GatewayRequestState, AuthenticatorError>
Poll the status of the World ID creation request.
Errors
- Will error if the network request fails.
- Will error if the gateway returns an error response.