pub enum AuthenticatorError {
Show 15 variants
PrimitiveError(PrimitiveError),
AccountDoesNotExist,
AccountAlreadyExists,
ContractError(Error),
NetworkError(Error),
PublicKeyNotFound,
GatewayError {
status: StatusCode,
body: String,
},
IndexerError {
status: StatusCode,
body: String,
},
Timeout,
InvalidConfig {
attribute: &'static str,
reason: String,
},
InvalidCredentialForProofRequest,
RegistrationError {
error_code: String,
error_message: String,
},
ProofError(ProofError),
InvalidIndexerPubkeySlot {
slot_index: usize,
max_supported_slot: usize,
},
Generic(String),
}Expand description
Errors that can occur when interacting with the Authenticator.
Variants
PrimitiveError(PrimitiveError)
Primitive error
AccountDoesNotExist
This operation requires a registered account and an account is not registered
for this authenticator. Call create_account first to register it.
AccountAlreadyExists
The account already exists for this authenticator. Call leaf_index to get the leaf index.
ContractError(Error)
An error occurred while interacting with the EVM contract.
NetworkError(Error)
Network/HTTP request error.
PublicKeyNotFound
Public key not found in the Authenticator public key set. Usually indicates the local state is out of sync with the registry.
GatewayError
Gateway returned an error response.
IndexerError
Indexer returned an error response.
Timeout
Account creation timed out while polling for confirmation.
InvalidConfig
Configuration is invalid or missing required values.
Fields
InvalidCredentialForProofRequest
The provided credential is not valid for the provided proof request.
RegistrationError
Error during the World ID registration process.
This usually occurs from an on-chain revert.
Fields
ProofError(ProofError)
Error on proof generation
InvalidIndexerPubkeySlot
Indexer returned an authenticator key slot that exceeds supported key capacity.
Fields
Generic(String)
Generic error for other unexpected issues.