pub enum ValidationError {
Show 16 variants
InvalidProofRequest(String),
RequestIdMismatch,
VersionMismatch,
ProofGenerationFailed(String),
MissingCredential(String),
UnexpectedCredential(String),
DuplicateCredential(String),
ConstraintNotSatisfied,
ConstraintTooDeep,
ConstraintTooLarge,
ExpiresAtMinMismatch(String, u64, u64),
SessionIdMismatch,
MissingSessionId,
UnexpectedSessionId,
MissingSessionNullifier(String),
MissingNullifier(String),
}Expand description
Validation errors when checking a response against a request
Variants
InvalidProofRequest(String)
The request’s proof type does not match its fields.
RequestIdMismatch
The response id does not match the request id
VersionMismatch
The response version does not match the request version
ProofGenerationFailed(String)
The proof generation failed (response contains an error)
MissingCredential(String)
A required credential was not provided
UnexpectedCredential(String)
A credential was returned that was not requested.
DuplicateCredential(String)
A credential identifier was returned more than once.
ConstraintNotSatisfied
The provided credentials do not satisfy the request constraints
ConstraintTooDeep
The constraints expression exceeds the supported nesting depth
ConstraintTooLarge
The constraints expression exceeds the maximum allowed size/complexity
ExpiresAtMinMismatch(String, u64, u64)
The expires_at_min value in the response does not match the expected value from the request
SessionIdMismatch
Session ID doesn’t match between request and response
MissingSessionId
Session ID missing from a create-session response.
UnexpectedSessionId
Session ID present in a uniqueness response.
MissingSessionNullifier(String)
Session nullifier missing for credential in session proof
MissingNullifier(String)
Nullifier missing for credential in uniqueness proof