pub struct Config { /* private fields */ }Expand description
Global configuration to interact with the different components of the Protocol.
Used by Authenticators and RPs.
Implementations
Sourceimpl Config
impl Config
Sourcepub fn new(
rpc_url: Option<String>,
chain_id: u64,
registry_address: Address,
indexer: ServiceEndpoint,
gateway: ServiceEndpoint,
nullifier_oracle_urls: Vec<String>,
nullifier_oracle_threshold: usize,
) -> Result<Config, PrimitiveError>
pub fn new( rpc_url: Option<String>, chain_id: u64, registry_address: Address, indexer: ServiceEndpoint, gateway: ServiceEndpoint, nullifier_oracle_urls: Vec<String>, nullifier_oracle_threshold: usize, ) -> Result<Config, PrimitiveError>
Sourcepub const fn chain_id(&self) -> u64
pub const fn chain_id(&self) -> u64
The chain ID of the network where the WorldIDRegistry contract is deployed.
Sourcepub const fn registry_address(&self) -> &Address
pub const fn registry_address(&self) -> &Address
The address of the WorldIDRegistry contract.
Sourcepub const fn indexer(&self) -> &ServiceEndpoint
pub const fn indexer(&self) -> &ServiceEndpoint
The indexer endpoint configuration. The indexer is used to fetch inclusion
proofs from the WorldIDRegistry contract.
Sourcepub const fn gateway(&self) -> &ServiceEndpoint
pub const fn gateway(&self) -> &ServiceEndpoint
The gateway endpoint configuration. The gateway is used to perform operations
on the WorldIDRegistry contract without leaking a wallet address.
Sourcepub fn indexer_url(&self) -> &str
pub fn indexer_url(&self) -> &str
Convenience accessor for the indexer’s target URL.
Sourcepub fn gateway_url(&self) -> &str
pub fn gateway_url(&self) -> &str
Convenience accessor for the gateway’s target URL.
Sourcepub const fn nullifier_oracle_urls(&self) -> &Vec<String>
pub const fn nullifier_oracle_urls(&self) -> &Vec<String>
The list of URLs of all and each node of the Nullifier Oracle.
Sourcepub const fn nullifier_oracle_threshold(&self) -> usize
pub const fn nullifier_oracle_threshold(&self) -> usize
The minimum number of Nullifier Oracle responses required to build a nullifier.