Authenticator

Struct Authenticator 

pub struct Authenticator {
    pub config: Config,
    pub packed_account_data: U256,
    /* private fields */
}
An Authenticator is the base layer with which a user interacts with the Protocol.

Fields

config: Config

General configuration for the Authenticator.

packed_account_data: U256

The packed account data for the holder’s World ID is a uint256 defined in the AccountRegistry contract as: recovery_counter (32 bits) | pubkey_id (commitment to all off-chain public keys) (32 bits) | leaf_index (192 bits)

Implementations

Source

impl Authenticator

pub async fn init( seed: &[u8], config: Config, ) -> Result<Self, AuthenticatorError>

Initialize an Authenticator from a seed and config.

This method will error if the World ID account does not exist on the registry.

Errors
  • Will error if the provided seed is invalid (not 32 bytes).
  • Will error if the RPC URL is invalid.
  • Will error if there are contract call failures.
  • Will error if the account does not exist (AccountDoesNotExist).
pub async fn init_or_create( seed: &[u8], config: Config, recovery_address: Option<Address>, ) -> Result<Option<Self>, AuthenticatorError>

Initialize an Authenticator from a seed and config, creating the account if it doesn’t exist.

If the account does not exist, it will automatically create it. Since account creation is asynchronous (requires on-chain transaction confirmation), this method will return None and you should poll.

Errors
  • See init for additional error details.
pub async fn init_or_create_blocking( seed: &[u8], config: Config, recovery_address: Option<Address>, ) -> Result<Self, AuthenticatorError>

Initialize an Authenticator from a seed and config, creating the account if it doesn’t exist and blocking until the account is confirmed on-chain.

Errors
  • Will error with Timeout if account creation takes longer than 30 seconds.
  • Will error if the gateway reports the account creation as failed.
  • See init for additional error details.
pub async fn get_packed_account_data( onchain_signer_address: Address, registry: Option<&AccountRegistryInstance<DynProvider>>, config: &Config, http_client: &Client, ) -> Result<U256, AuthenticatorError>

Returns the packed account data for the holder’s World ID.

The packed account data is a 256 bit integer which includes the World ID’s leaf index, their recovery counter, and their pubkey id/commitment.

Errors

Will error if the network call fails or if the account does not exist.

pub const fn onchain_address(&self) -> Address

Returns the k256 public key of the Authenticator signer which is used to verify on-chain operations, chiefly with the AccountRegistry contract.

pub fn offchain_pubkey(&self) -> EdDSAPublicKey

Returns the EdDSA public key of the Authenticator signer which is used to verify off-chain operations. For example, the Nullifier Oracle uses it to verify requests for nullifiers.

pub fn offchain_pubkey_compressed(&self) -> Result<U256, AuthenticatorError>

Returns the compressed EdDSA public key of the Authenticator signer which is used to verify off-chain operations. For example, the Nullifier Oracle uses it to verify requests for nullifiers.

Errors

Will error if the public key cannot be serialized.

pub fn registry(&self) -> Option<Arc<AccountRegistryInstance<DynProvider>>>

Returns a reference to the AccountRegistry contract instance.

pub fn leaf_index(&self) -> U256

Returns the account index for the holder’s World ID.

This is the index at the Merkle tree where the holder’s World ID account is registered.

pub fn recovery_counter(&self) -> U256

Returns the recovery counter for the holder’s World ID.

The recovery counter is used to efficiently invalidate all the old keys when an account is recovered.

pub fn pubkey_id(&self) -> U256

Returns the pubkey id (or commitment) for the holder’s World ID.

This is a commitment to all the off-chain public keys that are authorized to act on behalf of the holder.

pub async fn fetch_inclusion_proof( &self, ) -> Result<(MerkleInclusionProof<TREE_DEPTH>, AuthenticatorPublicKeySet), AuthenticatorError>

Fetches a Merkle inclusion proof for the holder’s World ID given their account index.

Errors
  • Will error if the provided indexer URL is not valid or if there are HTTP call failures.
  • Will error if the user is not registered on the registry.
pub async fn signing_nonce(&self) -> Result<U256, AuthenticatorError>

Returns the signing nonce for the holder’s World ID.

Errors

Will return an error if the registry contract call fails.

pub async fn generate_proof( &self, message_hash: FieldElement, rp_request: RpRequest, credential: Credential, ) -> Result<(Proof<Bn254>, FieldElement), AuthenticatorError>

Generates a World ID Uniqueness Proof given a provided context.

Errors
  • Will error if the any of the provided parameters are not valid.
  • Will error if any of the required network requests fail.
  • Will error if the user does not have a registered World ID.
pub async fn insert_authenticator( &mut self, new_authenticator_pubkey: EdDSAPublicKey, new_authenticator_address: Address, ) -> Result<String, AuthenticatorError>

Inserts a new authenticator to the account.

Errors

Will error if the provided RPC URL is not valid or if there are HTTP call failures.

Note

TODO: After successfully inserting an authenticator, the packed_account_data should be refreshed from the registry to reflect the new pubkey_id commitment.

pub async fn update_authenticator( &mut self, old_authenticator_address: Address, new_authenticator_address: Address, new_authenticator_pubkey: EdDSAPublicKey, index: u32, ) -> Result<String, AuthenticatorError>

Updates an existing authenticator slot with a new authenticator.

Errors

Returns an error if the gateway rejects the request or a network error occurs.

Note

TODO: After successfully updating an authenticator, the packed_account_data should be refreshed from the registry to reflect the new pubkey_id commitment.

pub async fn remove_authenticator( &mut self, authenticator_address: Address, index: u32, ) -> Result<String, AuthenticatorError>

Removes an authenticator from the account.

Errors

Returns an error if the gateway rejects the request or a network error occurs.

Note

TODO: After successfully removing an authenticator, the packed_account_data should be refreshed from the registry to reflect the new pubkey_id commitment.

pub fn leaf_hash(key_set: &AuthenticatorPublicKeySet) -> Fq

Computes the Merkle leaf (i.e. the commitment to the public key set) for a given public key set.

TODO: move to primitives

Errors

Will error if the provided public key set is not valid.